@@ -17,7 +17,7 @@ process_line_lwd_vector <- function(x, g, rescale) {
 	if (is.null(g$line.lwd.legend.labels)) {
 		line.lwd.legend.labels <- do.call("fancy_breaks", c(list(vec=w_legend, intervals=FALSE), g$legend.format))
 	} else {
-		if (length(g$line.lwd.legend.labels) != length(w_legend)) stop("length of sizes.legend.labels is not equal to the number of bubbles in the legend")
+		if (length(g$line.lwd.legend.labels) != length(w_legend)) stop("length of sizes.legend.labels is not equal to the number of bubbles in the legend", call. = FALSE)
 		line.lwd.legend.labels <- g$line.lwd.legend.labels
 	}
 	
@@ -29,53 +29,8 @@ process_line_lwd_vector <- function(x, g, rescale) {
 		 line.lwd.legend.labels=line.lwd.legend.labels)
 }
 
-process_line_col_vector <- function(x, g, gt) {
-	line.col.is.numeric <- is.numeric(x)
-	if (line.col.is.numeric) {
-		is.diverging <-  use_diverging_palette(x, g$breaks)
-		palette <- if (is.null(g$palette)) {
-			gt$aes.palette[[ifelse(is.diverging, "div", "seq")]] 
-		} else if (g$palette[1] %in% c("seq", "div", "cat")) {
-			gt$aes.palette[[g$palette[1]]]
-		} else g$palette
-		colsLeg <- num2pal(x, g$n, style=g$style, breaks=g$breaks, 
-						   palette = palette,
-						   auto.palette.mapping = g$auto.palette.mapping,
-						   contrast = g$contrast, legend.labels=g$labels,
-						   legend.NA.text=g$textNA,
-						   process.colors=c(list(alpha=g$alpha), gt$pc),
-						   legend.format=g$legend.format)
-		line.breaks <- colsLeg[[4]]
-	} else {
-		palette <- if (is.null(g$palette)) {
-			gt$aes.palette[[ifelse(is.ordered(x), "seq", "cat")]] 
-		} else if (g$palette[1] %in% c("seq", "div", "cat")) {
-			gt$aes.palette[[g$palette[1]]]
-		} else g$palette
-		#remove unused levels in legend
-		colsLeg <- cat2pal(x,
-						   palette = palette,
-						   contrast = g$contrast,
-						   colorNA = g$colorNA,
-						   legend.labels=g$labels,
-						   legend.NA.text=g$textNA,
-						   max_levels=g$max.categories,
-						   process.colors=c(list(alpha=g$alpha), gt$pc))
-		line.breaks <- NA
-	}
-	line.col <- colsLeg[[1]]
-	line.col.legend.labels <- colsLeg[[2]]
-	line.col.legend.palette <- colsLeg[[3]]
-	
-	list(line.col=line.col,
-		 line.col.legend.labels=line.col.legend.labels,
-		 line.col.legend.palette=line.col.legend.palette,
-		 line.col.is.numeric=line.col.is.numeric,
-		 line.breaks=line.breaks)
-	
-}
 
-process_lines <- function(data, g, gt, gby, z) {
+process_lines <- function(data, g, gt, gby, z, allow.small.mult) {
 	npol <- nrow(data)
 	by <- data$GROUP_BY
 	shpcols <- names(data)[1:(ncol(data)-1)]
@@ -87,9 +42,18 @@ process_lines <- function(data, g, gt, gby, z) {
 	xcol <- g$col
 	xlwd <- g$lwd
 	
-	if (is.na(xcol[1])) xcol <- gt$aes.colors["lines"]
-	if (is.na(g$colorNA)[1]) g$colorNA <- gt$aes.colors["na"]
+	if (!allow.small.mult) xcol <- xcol[1]
+	if (!allow.small.mult) xlwd <- xlwd[1]
 	
+	if (is.na(xcol[1])) xcol <- gt$aes.colors["lines"]
+	if (is.null(g$colorNA)) g$colorNA <- "#00000000"
+	if (is.na(g$colorNA)[1]) g$colorNA <- gt$aes.colors["na"]
+	if (g$colorNA=="#00000000") g$showNA <- FALSE
+	
+	if (!is.na(g$alpha) && !is.numeric(g$alpha)) stop("alpha argument in tm_lines is not a numeric", call. = FALSE)
+	
+	
+		
 	if (nlevels(by)>1) {
 		xcol <- xcol[1]
 		xlwd <- xlwd[1]
@@ -106,16 +70,17 @@ process_lines <- function(data, g, gt, gby, z) {
 	if (nxlwd<nx) xlwd <- rep(xlwd, length.out=nx)
 
 	if (!varylwd) {
-		if (!all(is.numeric(xlwd))) stop("Line widths are neither numeric nor valid variable name(s)")
+		if (!all(is.numeric(xlwd))) stop("Line widths are neither numeric nor valid variable name(s)", call. = FALSE)
 		for (i in 1:nx) data[[paste("lwd", i, sep="_")]] <- xlwd[i]
 		xlwd <- paste("lwd", 1:nx, sep="_")
 		gby$free.scales.line.lwd <- FALSE
-	}
+		split.by <- FALSE
+	} else split.by <- TRUE
 	
 	# check for direct color input
 	is.colors <- all(valid_colors(xcol))
 	if (!varycol) {
-		if (!is.colors) stop("Invalid line colors")
+		if (!is.colors) stop("Invalid line colors", call. = FALSE)
 		xcol <- do.call("process_color", c(list(col=col2hex(xcol), alpha=g$alpha), gt$pc))
 		for (i in 1:nx) data[[paste("COLOR", i, sep="_")]] <- xcol[i]
 		xcol <- paste("COLOR", 1:nx, sep="_")
@@ -124,18 +89,19 @@ process_lines <- function(data, g, gt, gby, z) {
 	nx <- max(nx, nlevels(by))
 	
 	dtcol <- process_data(data[, xcol, drop=FALSE], by=by, free.scales=gby$free.scales.line.col, is.colors=is.colors)
-	dtlwd <- process_data(data[, xlwd, drop=FALSE], by=by, free.scales=gby$free.scales.line.lwd, is.colors=FALSE)
+	dtlwd <- process_data(data[, xlwd, drop=FALSE], by=by, free.scales=gby$free.scales.line.lwd, is.colors=FALSE, split.by=split.by)
 	
+	if (nlevels(by)>1) if (is.na(g$showNA)) g$showNA <- attr(dtcol, "anyNA")
 	if (is.list(dtlwd)) {
 		# multiple variables for lwd are defined
 		gsl <- split_g(g, n=nx)
-		if (!all(sapply(dtlwd, is.numeric))) stop("lwd argument of tm_lines contains a non-numeric variable")
+		if (!all(sapply(dtlwd, is.numeric))) stop("lwd argument of tm_lines contains a non-numeric variable", call. = FALSE)
 		res <- mapply(process_line_lwd_vector, dtlwd, gsl, MoreArgs = list(rescale=varylwd), SIMPLIFY = FALSE)
 		line.lwd <- sapply(res, function(r)r$line.lwd)
 		line.legend.lwds <- lapply(res, function(r)r$line.legend.lwds)
 		line.lwd.legend.labels <- lapply(res, function(r)r$line.lwd.legend.labels)
 	} else {
-		if (!is.numeric(dtlwd)) stop("lwd argument of tm_lines is not a numeric variable")
+		if (!is.numeric(dtlwd)) stop("lwd argument of tm_lines is not a numeric variable", call. = FALSE)
 		res <- process_line_lwd_vector(dtlwd, g, rescale=varylwd)
 		line.lwd <- matrix(res$line.lwd, nrow=npol)
 		if (varylwd) {
@@ -150,13 +116,19 @@ process_lines <- function(data, g, gt, gby, z) {
 		}
 	}
 	
-	
+	# selection: which line widths are NA?
 	sel <- if (is.list(dtlwd)) {
 		lapply(dtlwd, function(i)!is.na(i))
-	} else !is.na(dtlwd)
-	
-	
+	} else {
+		if (is.list(dtcol)) {
+			cnts <- vapply(dtcol, length, integer(1))
+			cnts2 <- 1:length(dtcol)
+			f <- factor(unlist(mapply(rep, cnts2, cnts, SIMPLIFY = FALSE)))
+			split(dtlwd, f = f)
+		} else !is.na(dtlwd)
+	} 
 	
+
 	dcr <- process_dtcol(dtcol, sel, g, gt, nx, npol)
 	if (dcr$is.constant) xcol <- rep(NA, nx)
 	col <- dcr$col
@@ -192,6 +164,10 @@ process_lines <- function(data, g, gt, gby, z) {
 		line.col.legend.hist.title <- g$legend.hist.title
 	} else line.col.legend.hist.title <- ""
 	
+	if (!g$legend.lwd.show) line.lwd.legend.title <- NA
+	if (!g$legend.col.show) line.col.legend.title <- NA
+	
+	
 	list(line.col=col,
 		 line.lwd=line.lwd,
 		 line.lty=g$lty,
@@ -219,7 +195,8 @@ process_lines <- function(data, g, gt, gby, z) {
 		 line.col.legend.hist.title=line.col.legend.hist.title,
 		 line.col.legend.z=line.col.legend.z,
 		 line.lwd.legend.z=line.lwd.legend.z,
-		 line.col.legend.hist.z=line.col.legend.hist.z
+		 line.col.legend.hist.z=line.col.legend.hist.z,
+		 line.id=g$id
 	)
 
 }
