@@ -35,11 +35,16 @@ split.SpatialLines <- function(x, f, drop=FALSE, ...) split_shape(x, f, drop=FAL
 
 split_shape <- function(x, f, drop=TRUE, ...) {
 	if (!is.factor(f)) {
-		warning("f is not a factor")
+		warning("f is not a factor", call. = FALSE)
 		f <- as.factor(f)
 	}
-	lev <- intersect(levels(f), f)
-	xlist <- lapply(lev, function(l)x[which(f==l),])
+	lev <- if (drop) {
+		intersect(levels(f), f)	
+	} else levels(f)
+	xlist <- lapply(lev, function(l) {
+		ids <- which(f==l)
+		if (length(ids)==0L) NULL else x[ids,]
+	})
 	names(xlist) <- lev
 	xlist
 }
\ No newline at end of file
